// MC9090_TestDlg.cpp : Implementierungsdatei
//

#include "stdafx.h"
#include "MC9090_Test.h"
#include "MC9090_TestDlg.h"
#include "feisc.h"
#include "fecom.h"

#define WM_USER_CTS						(WM_USER + 5)
#define WM_USER_RTS						(WM_USER + 6)
#define WM_USER_DEVICE_DISCONNECT		(WM_USER + 7)
#define WM_USER_DEVICE_CONNECT			(WM_USER + 8)

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

// CMC9090_TestDlg-Dialogfeld

CMC9090_TestDlg::CMC9090_TestDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CMC9090_TestDlg::IDD, pParent)
	, m_bCTS(FALSE)
{
	int iReaderHnd = 0;

	m_iPortHnd = 0;
	m_iReaderHnd = 0;

	iReaderHnd = FEISC_NewReader(0);
	if(iReaderHnd > 0)
		m_iReaderHnd = iReaderHnd;

	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CMC9090_TestDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Check(pDX, IDC_CHK_CTS, m_bCTS);
	DDX_Control(pDX, IDC_EVENT_LIST, m_EventList);
}

BEGIN_MESSAGE_MAP(CMC9090_TestDlg, CDialog)
#if defined(_DEVICE_RESOLUTION_AWARE) && !defined(WIN32_PLATFORM_WFSP)
	ON_WM_SIZE()
#endif
	//}}AFX_MSG_MAP
	ON_BN_CLICKED(IDC_BTN_OPEN, &CMC9090_TestDlg::OnBnClickedBtnOpen)
	ON_MESSAGE(WM_USER_CTS, OnCTSMessage)
	ON_MESSAGE(WM_USER_RTS, OnRTSMessage)
	ON_MESSAGE(WM_USER_DEVICE_CONNECT, OnConnect)
	ON_MESSAGE(WM_USER_DEVICE_DISCONNECT, OnDisConnect)
	ON_BN_CLICKED(IDC_BTN_0x65, &CMC9090_TestDlg::OnBnClickedBtn0x65)
END_MESSAGE_MAP()


// CMC9090_TestDlg-Meldungshandler

BOOL CMC9090_TestDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Symbol fr dieses Dialogfeld festlegen. Wird automatisch erledigt
	//  wenn das Hauptfenster der Anwendung kein Dialogfeld ist
	SetIcon(m_hIcon, TRUE);			// Groes Symbol verwenden
	SetIcon(m_hIcon, FALSE);		// Kleines Symbol verwenden

	// TODO: Hier zustzliche Initialisierung einfgen
	
	return TRUE;  // Geben Sie TRUE zurck, auer ein Steuerelement soll den Fokus erhalten
}

#if defined(_DEVICE_RESOLUTION_AWARE) && !defined(WIN32_PLATFORM_WFSP)
void CMC9090_TestDlg::OnSize(UINT /*nType*/, int /*cx*/, int /*cy*/)
{
	if (AfxIsDRAEnabled())
	{
		DRA::RelayoutDialog(
			AfxGetResourceHandle(), 
			this->m_hWnd, 
			DRA::GetDisplayMode() != DRA::Portrait ? 
			MAKEINTRESOURCE(IDD_MC9090_TEST_DIALOG_WIDE) : 
			MAKEINTRESOURCE(IDD_MC9090_TEST_DIALOG));
	}
}
#endif


void CMC9090_TestDlg::OnBnClickedBtnOpen()
{
	int iError = 0;
	int iPortHnd = 0;
	char cPortHnd[16];

	if(m_iPortHnd > 0)
	{
		FECOM_ClosePort(m_iPortHnd);
		GetDlgItem(IDC_BTN_OPEN)->SetWindowTextW(_T("OpenPort"));
		m_iPortHnd = 0;

		if(m_iReaderHnd > 0)
		{
			FEISC_SetReaderPara(m_iReaderHnd, "PortHnd", "0");
		}

		return;
	}

	// open the port
	iPortHnd = FECOM_OpenPort("1");
	if(iPortHnd < 0)
		return;

	// set port parameters
	FECOM_SetPortPara(iPortHnd, "Baud", "38400");
	FECOM_SetPortPara(iPortHnd, "Frame", "8E1");
	FECOM_SetPortPara(iPortHnd, "Timeout", "1000");

	// install event handler for port close messages
	m_Init.uiFlag	= FECOM_WND_HWND;
	m_Init.uiUse	= FECOM_PORT_CLOSED_EVENT;
	m_Init.uiMsg	= WM_USER_DEVICE_DISCONNECT;
	m_Init.hwndWnd	= this->GetSafeHwnd();
	iError = FECOM_AddEventHandler(iPortHnd, &m_Init);
	if (iError < 0)
	{
		FECOM_ClosePort(iPortHnd);
		return;
	}

	// install event handler for port open messages
	m_Init.uiFlag	= FECOM_WND_HWND;
	m_Init.uiUse	= FECOM_PORT_REOPEN_EVENT;
	m_Init.uiMsg	= WM_USER_DEVICE_CONNECT;
	m_Init.hwndWnd	= this->GetSafeHwnd();
	iError = FECOM_AddEventHandler(iPortHnd, &m_Init);
	if (iError < 0)
	{
		FECOM_ClosePort(iPortHnd);
		return;
	}

	// install event handler for CTS
	m_Init.uiFlag = FECOM_WND_HWND;
	m_Init.uiUse = FECOM_CTS_EVENT;
	m_Init.uiMsg = WM_USER_CTS;
	m_Init.hwndWnd = this->GetSafeHwnd();
	iError = FECOM_AddEventHandler(iPortHnd, &m_Init);
	if (iError < 0)
	{
		FECOM_ClosePort(iPortHnd);
		return;
	}

	// install event handler for RTS
	m_Init.uiFlag = FECOM_WND_HWND;
	m_Init.uiUse = FECOM_RTS_EVENT;
	m_Init.uiMsg = WM_USER_RTS;
	m_Init.hwndWnd = this->GetSafeHwnd();
	iError = FECOM_AddEventHandler(iPortHnd, &m_Init);
	if (iError < 0)
	{
		FECOM_ClosePort(iPortHnd);
		return;
	}

	if(m_iReaderHnd > 0)
	{
		sprintf(cPortHnd, "%d", iPortHnd);
		FEISC_SetReaderPara(m_iReaderHnd, "PortHnd", cPortHnd);
		FEISC_SetReaderPara(m_iReaderHnd, "PowerOnTime", "5"); // example to modify the power on time
	}

	GetDlgItem(IDC_BTN_OPEN)->SetWindowTextW(_T("ClosePort"));
	m_iPortHnd = iPortHnd;
}

LRESULT CMC9090_TestDlg::OnCTSMessage(WPARAM wParam,LPARAM lParam)
{
	if(wParam == 1)
	{
		if(lParam == 0)
			m_bCTS = FALSE;
		else
			m_bCTS = TRUE;

		UpdateData(FALSE);
	}

	return TRUE;
}

LRESULT CMC9090_TestDlg::OnRTSMessage(WPARAM wParam,LPARAM lParam)
{
	if(wParam == 1)
	{
		if(lParam == 0)
		{
			m_EventList.AddString(_T("Power Down"));
			m_EventList.SetTopIndex(m_EventList.GetCount() - 1);
		}
		else
		{
			m_EventList.AddString(_T("Power Up"));
			m_EventList.SetTopIndex(m_EventList.GetCount() - 1);
		}

		UpdateData(FALSE);
	}

	return TRUE;
}

LRESULT CMC9090_TestDlg::OnConnect(WPARAM wParam,LPARAM lParam)
{
	m_EventList.AddString(_T("Open Port"));
	m_EventList.SetTopIndex(m_EventList.GetCount() - 1);
	return TRUE;
}

LRESULT CMC9090_TestDlg::OnDisConnect(WPARAM wParam,LPARAM lParam)
{
	m_EventList.AddString(_T("Close Port"));
	m_EventList.SetTopIndex(m_EventList.GetCount() - 1);
	return TRUE;
}

void CMC9090_TestDlg::OnBnClickedBtn0x65()
{
	unsigned char cVersion[64];
	char cErrorText[256];
	CString sMsg;

	int iError = FEISC_0x65_SoftVersion( m_iReaderHnd, 0, &cVersion[0], 1 );
	if(iError < 0)
	{
		if( iError > -1100 && iError <= -1000 )
		{
			FECOM_GetErrorText(iError, cErrorText);
			sMsg.Format(_T("%S"), cErrorText);
			m_EventList.AddString(sMsg);
			m_EventList.SetTopIndex(m_EventList.GetCount() - 1);
		}
		else if( iError > -4100 && iError < -4000 )
		{
			FEISC_GetErrorText(iError, cErrorText);
			sMsg.Format(_T("%S"), cErrorText);
			m_EventList.AddString(sMsg);
			m_EventList.SetTopIndex(m_EventList.GetCount() - 1);
		}
	}
	else if(iError > 0)
	{
		FEISC_GetStatusText(iError, cErrorText);
		sMsg.Format(_T("%S"), cErrorText);
		m_EventList.AddString(sMsg);
		m_EventList.SetTopIndex(m_EventList.GetCount() - 1);
	}
	else
	{
		sMsg.Format(_T("%S"), cVersion);
		m_EventList.AddString(sMsg);
		m_EventList.SetTopIndex(m_EventList.GetCount() - 1);
	}
}
